from pathlib import Path

from django.urls import reverse_lazy
from django.contrib import messages

from autosecretkey import AutoSecretKey
from reportmonster.classes.config import MonsterConfig

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

CONFIG_PATH = BASE_DIR / "settings.ini"

ASK = AutoSecretKey(CONFIG_PATH)
MONSTERCONFIG = MonsterConfig(CONFIG_PATH)

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ASK.secret_key

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = ASK.config.getboolean("ACADEMON", "Debug", fallback=False)
CRISPY_FAIL_SILENTLY = not DEBUG

ADMINS = [(ASK.config.get("ACADEMON", "AdminName"), ASK.config.get("ACADEMON", "AdminEmail"))]

ALLOWED_HOSTS = [ASK.config["ACADEMON"]["Host"]]
CSRF_TRUSTED_ORIGINS = [f"https://{host}" for host in ALLOWED_HOSTS]

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'ajax_datatable',
    'crispy_forms',
    'dbsettings',
    'core',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'academon.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / "templates"],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'academon.wsgi.application'


# Database
# https://docs.djangoproject.com/en/4.0/ref/settings/#databases

if "MySQL" in ASK.config:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.mysql',
            'NAME': ASK.config.get("MySQL", "Database"),
            'USER': ASK.config.get("MySQL", "Username"),
            'PASSWORD': ASK.config.get("MySQL", "Password"),
            'HOST': ASK.config.get("MySQL", "Host", fallback="localhost"),
            'PORT': ASK.config.getint("MySQL", "Port", fallback=3306)
        }
    }

else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': BASE_DIR / 'db.sqlite3',
        }
    }


# Password validation
# https://docs.djangoproject.com/en/4.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.0/howto/static-files/

STATIC_URL = '/static/'

STATIC_ROOT = None if DEBUG else ASK.config.get(
    "ACADEMON", "StaticRoot", fallback=None)

STATICFILES_DIRS = [
    BASE_DIR / "static",
]

if "S3" in ASK.config:
    DEFAULT_FILE_STORAGE = 'storages.backends.s3boto3.S3Boto3Storage'
    STATICFILES_STORAGE = 'storages.backends.s3boto3.S3StaticStorage'
    AWS_ACCESS_KEY_ID = ASK.config.get("S3", "AccessKey")
    AWS_SECRET_ACCESS_KEY = ASK.config.get("S3", "SecretKey")
    AWS_STORAGE_BUCKET_NAME = ASK.config.get("S3", "Bucket")
    AWS_S3_ENDPOINT_URL = ASK.config.get("S3", "Endpoint")

# Default primary key field type
# https://docs.djangoproject.com/en/4.0/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

AUTH_USER_MODEL = 'core.User'

# ReportMonster integration

AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
]

if ASK.config.getboolean("REPORTMONSTER", "Login"):
    LOGIN_VESSEL = ASK.config["REPORTMONSTER"]["LoginVessel"]
    AUTHENTICATION_BACKENDS.append('core.backends.ReportMonsterBackend')
else:
    LOGIN_VESSEL = None

# Login System

LOGIN_URL = reverse_lazy("core:login")
LOGIN_REDIRECT_URL = reverse_lazy("core:dashboard")
LOGOUT_REDIRECT_URL = reverse_lazy("core:index")

# Logging

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
        },
    },
    'root': {
        'handlers': ['console'],
        'level': 'DEBUG',
    },
}

# Messages

MESSAGE_TAGS = {
    messages.DEBUG: 'alert-info',
    messages.INFO: 'alert-info',
    messages.SUCCESS: 'alert-success',
    messages.WARNING: 'alert-warning',
    messages.ERROR: 'alert-danger',
}